#!/bin/sh
#
# Usage
#   $ binaries-search-ldd.sh libfoo ...
#	- find binaries (executable, libraries) and list dynamic dependencies
#	only matching with libfoo, from system directories.
#   $ installed-binaries-search-ldd.sh libfoo ... < basename.installed ...
#	- find binaries (executable, libraries) and list dynamic dependencies
#	only matching with libfoo, from file-list standard input.
#
# Author(s)
#   Taiji Yamada <taiji@aihara.co.jp>
#
LC_ALL=C
export LC_ALL
case "`uname -s`" in
Darwin)
  file='file -b'
  ldd='otool -L'
  fgrepldd(){
    $ldd "$2" | fgrep "$1" > /dev/null 2> /dev/null && {
      $ldd "$2" | fgrep "$1" | sed -ne "s|[	 ]*\(.*\) \((.*)\)|\1	\2	$2|p"
    }
  }
  ;;
SunOS|*)
  file='file'
  ldd='lddforall'
  fgrepldd(){
    ldd "$2" | fgrep "$1" > /dev/null 2> /dev/null && {
      ldd "$2" | fgrep "$1" | sed -ne "s|[	 ]*\(.*\) =>[	 ]*\(.*\)|\2	\1	$2|p"
    }
  }
  ;;
esac

lists=false
case "`basename \"$0\"`" in
installed-*)	lists=:	;;
esac

lddforall(){
  for f in "$@"; do
    echo "$f:"
    ldd "$f"
  done
}
listup_binaries(){
  { $lists && cat || find "${@-.}" -type f; } | while read o; do
    case "$o" in
    *)
      t="`$file \"$o\"`"
      case "$t" in
      *"ELF "*" executable"*" dynamically linked"*)	;;
      *"Mach-O"*" executable"*)	;;
#      *"current ar archive"*)	;;
#      *"ar archive random library"*)	;;
      *"ELF "*" dynamic lib"*)	;;
      *"Mach-O"*" dynamically linked shared library"*)	;;
      *"Mach-O"*" bundle"*)	;;
      *) continue	;;
      esac
      ;;
    esac
    echo "$o"
  done
}
listup_binaries /opt/local/bin /opt/local/lib | while read b; do
  i=1
  while [ $i -le $# ]; do
    eval a="\${$i}"
    fgrepldd "$a" "$b"
    i=`expr $i + 1`
  done
done
