#!/bin/ksh
#
# Usage
#   $ make-installed.sh [-s] [-d]
#	- do `make install install.man`
#	with logging to ../.`basename $PWD`-codename-install.log and
#	output list of installed files to ../`basename $PWD`-codename.installed
#
#   $ make-installed.sh [-s] [-d] args ...
#	- do `args ...`
#	with logging to ../.`basename $PWD`-codename-install.log,
#	output list of installed files to ../`basename $PWD`-codename.installed
#	and arguments are written into ../`basename $PWD`-codename-install.sh
#
#   Installed files are found by `find -newer` touched file before installing.
#   In the case an installation by `cp -p` like emacs-lisp, this script can
#   find all installed files with '-d' option listing installed data files to
#   ../`basename $PWD`-data-codename.installed
#   If your environment have sudo, it does sudo what to do with '-s' option.
#
# Author(s)
#   Taiji Yamada <taiji@aihara.co.jp>
#

codename=${CODENAME:-`uname -n | sed -ne 's/\([^.]*\).*/\1/p' | tr A-Z a-z`}
installed_data=false
while [ "$1" != "" ]; do case "$1" in
-s) sudo=sudo	;;
-d) installed_data=:	;;
*) break	;;
esac; shift; done

pwd=`pwd`
basename=`basename "$pwd"`
touch="../.$basename-$codename.touch"
touched="../.$basename-$codename.touched"
log="../.$basename-$codename-install.log"
install="../$basename-$codename-install.sh"
installed="../$basename-$codename.installed"
$installed_data && {
data_installed="../$basename-data-$codename.installed"
touch_list="../.$basename-$codename.touch_list"
touched_list="../.$basename-$codename.touched_list"
all_installed="../.$basename-$codename.all_installed"
}

find_extra_args='! -regex .*/info/dir$ ! -name charset.alias ! -name locale.alias ! -name app-defaults'

keep_file(){
  keep_file="$1"
  kept_file=
  if [ -f "$keep_file" ]; then
    kept_file="${keep_file}~"
    while [ -f "$kept_file" ]; do kept_file="${kept_file}~"; done
    echo "mv $keep_file $kept_file"
    mv "$keep_file" "$kept_file"
  fi
}

LC_ALL=C
export LC_ALL

touch "$touch"
$installed_data && find /usr/local/* /usr/X11R*/* /etc/X11/* /usr/openwin/* /opt/local/* $find_extra_args ! -type d > "$touch_list"
sleep 2

if [ "$#" = 0 ]; then
  if [ -f "$install" ]; then
    $sudo sh "$install" 2>&1 | tee "$log"
  elif [ -f Imakefile ]; then
    $sudo make install install.man 2>&1 | tee "$log"
  else
    $sudo make install 2>&1 | tee "$log"
  fi
else
  keep_file "$install"
  (for v in "$@"; do
    echo "$v"|fgrep ' '>/dev/null && printf "'%s'" "$v" || printf "%s" "$v"
    printf " "
  done; echo "") | tee "$install"
  $sudo "$@" 2>&1 | tee "$log"
fi

$installed_data && find /usr/local/* /usr/X11R*/* /etc/X11/* /usr/openwin/* /opt/local/* $find_extra_args ! -type d > "$touched_list"
sleep 2
touch "$touched"

keep_file "$installed"
$sudo find /usr/local/* /usr/X11R*/* /etc/X11/* /usr/openwin/* /opt/local/* -newer "$touch" ! -newer "$touched" \( ! -type d -o \( -type d \( -empty -o \( ! -user root -o ! -perm u+w,a+rx \) \) \) \) $find_extra_args | tee "$installed"
$installed_data && {
  keep_file "$data_installed"
  diff -u "$touch_list" "$touched_list" | grep '^+[^+]' | sed 's|^+||' | grep -v '/$' > "$all_installed"
  diff -u "$installed" "$all_installed" | grep '^+[^+]' | sed 's|^+||' | grep -v '/$' > "$data_installed"
  rm "$touch_list" "$touched_list" "$all_installed"
  [ -s "$data_installed" ] || rm "$data_installed"
}
