#!/bin/ksh
#
# Usage
#   $ make-logging.sh ./configure
#   $ make-logging.sh xmkmf -a
#	- do `args ...`
#	with logging to ../.`basename $PWD`-codename-make.log,
#	and arguments are written into ../`basename $PWD`-codename-build.sh
#
#   $ make-logging.sh -a make
#   $ make-logging.sh -a make CC=gcc CFLAGS=-Wall
#	- do `args ...` following "-a" option
#	with logging to ../.`basename $PWD`-codename-build.log and
#	and arguments are add into ../`basename $PWD`-codename-build.sh
#
#   $ make-logging.sh [-b|-c]
#	- do sh ../`basename $PWD`-codename-build.sh, and rename the logfile
#	../.`basename $PWD`-codename-build.log if '-b' option is specified,
#	otherwise delete the logfile if '-c' option is specified.
#
# Author(s)
#   Taiji Yamada <taiji@aihara.co.jp>
#

codename=${CODENAME:-`uname -n | sed -ne 's/\([^.]*\).*/\1/p' | tr A-Z a-z`}
build_append=false; log_backup=false; log_clear=false
while [ "$1" != "" ]; do case "$1" in
-a) build_append=:	;;
-b) log_backup=:	;;
-c) log_clear=:	;;
*) break	;;
esac; shift; done

pwd=`pwd`
basename=`basename "$pwd"`
log="../.$basename-$codename-build.log"
build="../$basename-$codename-build.sh"

keep_file(){
  keep_file="$1"
  kept_file=
  if [ -f "$keep_file" ]; then
    kept_file="${keep_file}~"
    while [ -f "$kept_file" ]; do kept_file="${kept_file}~"; done
    echo "mv $keep_file $kept_file"
    mv "$keep_file" "$kept_file"
  fi
}

LC_ALL=C
export LC_ALL

$log_clear && [ -f "$log" ] && { echo rm "$log"; rm "$log"; }
$log_backup && keep_file "$log"

if [ "$#" = 0 -a -f "$build" ]; then
  sh "$build" 2>&1 | tee -a "$log"
  #sh "$build" >> "$log" 2>&1
else
  [ "$build_append" != false ] && tee="tee -a" || tee=tee
  [ "$build_append" != false ] || keep_file "$build"
  (for v in "$@"; do
    echo "$v"|fgrep ' '>/dev/null && printf "'%s'" "$v" || printf "%s" "$v"
    printf " "
  done; echo "") | $tee "$build"
  "$@" 2>&1 | $tee "$log"
  #[ "$build_append" != false ] || rm "$log"; "$@" >> "$log" 2>&1
fi
