#!/bin/ksh
#
# Usage
#   $ make-update.sh pathname_old pathname_new
#	- create new scripts to build from old scripts as templates
#   $ make-translate.sh pathname-codename_old pathname-codename_new
#	- create new scripts to build from other scripts as templates
#
# Author(s)
#   Taiji Yamada <taiji@aihara.co.jp>
#

codename=${CODENAME:-`uname -n | sed -ne 's/\([^.]*\).*/\1/p' | tr A-Z a-z`}

case $# in
0|1)
  echo "make-update.sh pathname_old pathname_new"
  echo "make-translate.sh pathname-codename_old pathname-codename_new"
  exit
  ;;
esac

case "`basename $0 .sh`" in
*-update)
  pathname[0]="$1"
  basename[0]="`basename \"$1\"`"
  prepare[0]="${pathname[0]}-$codename-prepare.sh"
  patch[0]="${pathname[0]}-$codename.patch"
  build[0]="${pathname[0]}-$codename-build.sh"
  install[0]="${pathname[0]}-$codename-install.sh"
  hostconfig[0]="${pathname[0]}-$codename-hostconfig.sh"
  hostunconfig[0]="${pathname[0]}-$codename-hostunconfig.sh"
  #
  pathname[1]="$2"
  basename[1]="`basename \"$2\"`"
  prepare[1]="${pathname[1]}-$codename-prepare.sh"
  patch[1]="${pathname[1]}-$codename.patch"
  build[1]="${pathname[1]}-$codename-build.sh"
  install[1]="${pathname[1]}-$codename-install.sh"
  hostconfig[1]="${pathname[1]}-$codename-hostconfig.sh"
  hostunconfig[1]="${pathname[1]}-$codename-hostunconfig.sh"
  ;;
*-translate)
  pathname[0]="$1"
  basename[0]="`basename \"$1\"`"
  building[0]="`basename \"$1\" | sed -ne 's/\(.*\)-\([^-]*\)$/\1/p'`"
  prepare[0]="${pathname[0]}-prepare.sh"
  patch[0]="${pathname[0]}.patch"
  build[0]="${pathname[0]}-build.sh"
  install[0]="${pathname[0]}-install.sh"
  hostconfig[0]="${pathname[0]}-hostconfig.sh"
  hostunconfig[0]="${pathname[0]}-hostunconfig.sh"
  #
  pathname[1]="$2"
  basename[1]="`basename \"$2\"`"
  building[1]="`basename \"$2\" | sed -ne 's/\(.*\)-\([^-]*\)$/\1/p'`"
  prepare[1]="${pathname[1]}-prepare.sh"
  patch[1]="${pathname[1]}.patch"
  build[1]="${pathname[1]}-build.sh"
  install[1]="${pathname[1]}-install.sh"
  hostconfig[1]="${pathname[1]}-hostconfig.sh"
  hostunconfig[1]="${pathname[1]}-hostunconfig.sh"
  ;;
esac

for v in prepare patch build install hostconfig hostunconfig; do
  eval "f0=\${$v[0]}; f1=\${$v[1]}"
  if [ -f "$f0" -a "$f0" != "$f1" -a ! -f "$f1" ]; then
    echo "$f1"
#    echo sed -e "s/${basename[0]}/${basename[1]}/g" "$f0" ">" "$f1"
    sed -e "s/${basename[0]}/${basename[1]}/g${building[0]:+;s/${building[0]}/${building[1]}/g}" "$f0" > "$f1"
  fi
done
