#!/bin/sh
#
# Usage
#   $ tbz2-installed.sh filename.tar.bz2 [filename.tar.bz2 ...]
#   $ tgz-installed.sh filename.tar.gz [filename.tar.gz ...]
#   $ txz-installed.sh filename.tar.xz [filename.tar.xz ...]
#   write contents list of the archive to filename-codename.installed
#
#   $ tbz2-install.sh filename.tar.bz2 [filename.tar.bz2 ...]
#   $ tgz-install.sh filename.tar.gz [filename.tar.gz ...]
#   $ txz-install.sh filename.tar.xz [filename.tar.xz ...]
#   do installation of archive into root directory and
#   write contents list of the archive to filename-codename.installed
#
#   $ tbz2-uninstall.sh filename.tar.bz2 [filename.tar.bz2 ...]
#   $ tgz-uninstall.sh filename.tar.gz [filename.tar.gz ...]
#   $ txz-uninstall.sh filename.tar.xz [filename.tar.xz ...]
#   write contents list of the archive to filename-codename.installed and
#   do uninstallation of archive from root directory by uninstall-*.sh
#
# Author(s)
#   Taiji Yamada <taiji@aihara.co.jp>
#

codename=${CODENAME:-`uname -n | sed -ne 's/\([^.]*\).*/\1/p' | tr A-Z a-z`}
uninstall=false
while true; do case "$1" in
-s) sudo=sudo	;;
*) break	;;
esac; shift; done

case $# in
0)
  echo "`basename $0` [-s] filename{.tar.gz|.tar.bz2|.tar.xz} [...]"
  exit
  ;;
esac

case `basename $0 .sh` in
tgz-*)
  AR=tar
  $AR --help | fgrep 'gnu.org' 2>&1 > /dev/null || AR="gnu$AR"
  FILTER="gzip -d -c"
  SUF=.tar.gz
  uninstall_ar_sh=uninstall-tgz.sh
  ;;
tbz2-*)
  AR=tar
  $AR --help | fgrep 'gnu.org' 2>&1 > /dev/null || AR="gnu$AR"
  FILTER="bzip2 -d -c"
  SUF=.tar.bz2
  uninstall_ar_sh=uninstall-tbz2.sh
  ;;
txz-*)
  AR=tar
  $AR --help | fgrep 'gnu.org' 2>&1 > /dev/null || AR="gnu$AR"
  FILTER="xz -d -c"
  SUF=.tar.xz
  uninstall_ar_sh=uninstall-txz.sh
  ;;
esac

case `basename $0 .sh` in
*-installed)
  ARFLAG="-t"
  ;;
*-install)
  ARFLAG="-C / -xv"
  ;;
*-uninstall)
  ARFLAG="-t"
  uninstall=:
  ;;
esac

keep_file(){
  keep_file="$1"
  kept_file=
  if [ -f "$keep_file" ]; then
    kept_file="${keep_file}~"
    while [ -f "$kept_file" ]; do kept_file="${kept_file}~"; done
    echo "mv $keep_file $kept_file"
    mv "$keep_file" "$kept_file"
  fi
}

for archive in "$@"; do
  dirname="`dirname \"$archive\"`"
  basename="`basename \"$archive\" $SUF`"
  installed="$dirname/$basename-$codename.installed"
  keep_file "$installed"
  echo "$FILTER $archive | $sudo $AR $ARFLAG -f - | sed 's|/$||' > $installed"
  $FILTER "$archive" | $sudo $AR $ARFLAG -f - | sed 's|/$||;s|^|/|;s|^//|/|' > "$installed"
  touch -r "$archive" "$installed"
  [ -f "$kept_file" ] && cmp "$kept_file" "$installed" > /dev/null 2>&1 && mv -f "$kept_file" "$installed"
  $uninstall && $uninstall_ar_sh ${sudo:+-s} "$installed"
done
