#!/bin/ksh
#
# Usage
#   $ installed-tbz2.sh basename.installed [basename.installed ...]
#	- archive files from basename.installed list into basename.tar.bz2
#   $ installed-tgz.sh basename.installed [basename.installed ...]
#	- archive files from basename.installed list into basename.tar.gz
#   $ installed-txz.sh basename.installed [basename.installed ...]
#	- archive files from basename.installed list into basename.tar.xz
#
#   $ uninstall-tbz2.sh basename.installed [basename.installed ...]
#	- archive files from basename.installed list into basename.tar.bz2
#	and delete all, be carefully!
#   $ uninstall-tgz.sh basename.installed [basename.installed ...]
#	- archive files from basename.installed list into basename.tar.gz
#	and delete all, be carefully!
#   $ uninstall-txz.sh basename.installed [basename.installed ...]
#	- archive files from basename.installed list into basename.tar.xz
#	and delete all, be carefully!
#
# Author(s)
#   Taiji Yamada <taiji@aihara.co.jp>
#

uninstall=false
while true; do case "$1" in
-s) sudo=sudo	;;
-C) shift; destdir="$1"	;;
*) break	;;
esac; shift; done

case $# in
0)
  echo "`basename $0` [-s] [-C destdir] basename.installed [basename2.installed ...]"
  exit
  ;;
esac

case `basename $0 .sh` in
*-tgz)
  AR="tar -C /"
  FILTER=gzip
  SUF=.tar.gz
  ;;
*-tbz2)
  AR="tar -C /"
  FILTER=bzip2
  SUF=.tar.bz2
  ;;
*-txz)
  AR="tar -C /"
  FILTER=xz
  SUF=.tar.xz
  ;;
esac

case `basename $0 .sh` in
uninstall-*)
  $AR --help | fgrep 'gnu.org' 2>&1 > /dev/null || AR="gnu$AR"
  AR="$AR --remove-files"
  uninstall=:
  ;;
esac

$AR --help | fgrep 'gnu.org' 2>&1 > /dev/null && AR="$AR --no-recursion" || AR="$AR -n"

keep_file(){
  keep_file="$1"
  kept_file=
  if [ -f "$keep_file" ]; then
    kept_file="${keep_file}~"
    while [ -f "$kept_file" ]; do kept_file="${kept_file}~"; done
    echo "mv $keep_file $kept_file"
    mv "$keep_file" "$kept_file"
  fi
}

for installed in "$@"; do
  [ "$destdir" = "" ] && dirname="`dirname \"$installed\"`" || dirname="$destdir"
  basename="`basename \"$installed\" .installed`"
  archive="$dirname/$basename-binaries$SUF"
  grep '/$' "$installed" > /dev/null 2>&1 && continue
  keep_file "$archive"
  echo "$AR --files-from=$installed -cvf - | $FILTER > $archive"
  $sudo $AR --files-from="$installed" -cvf - | $FILTER > "$archive"
  $uninstall && cat "$installed" | while read p; do
    [ -d "$p" ] && $sudo find "$p" -depth -type d -empty -exec rmdir {} \;
  done
  touch -r "$installed" "$archive"
  [ -f "$kept_file" ] && cmp "$kept_file" "$archive" > /dev/null 2>&1 && mv -f "$kept_file" "$archive"
  $uninstall && case "$installed" in
  *.installed)
    uninstalled="`dirname \"$installed\"`/`basename \"$installed\" .installed`.uninstalled"
    keep_file "$uninstalled"
    mv "$installed" "$uninstalled"
    ;;
  esac
done
