#!/bin/sh
#
# Usage
#   $ make-patches [-a|-b] [-p]
#	- make ../`basename $PWD`-codename.patch file by directory comparing
#	with current directory if ../`basename $PWD`.orig directory exists,
#	otherwise find original files such as *~, *.org and *.orig, and
#	make ../`basename $PWD`-codename.patch file by files comparing.
#
#   $ make-patches [-a|-b] [-p] ../dirname
#	- make ../`basename $PWD`-codename.patch file by directory comparing
#	../dirname directory and current directory if ../dirname exists,
#	otherwise find original files such as *~, *.org and *.orig and
#	make ../`basename $PWD`-codename.patch file by files comparing.
#
#	"-a" option appends comparing output to the patch file.
#	"-b" option renames the patch file already exists.
#	"-p" option appends patching to ../`basename $PWD`-codename-prepare.sh
#
#   $ view-patches [-a]
#   $ view-patches [-a] ../dirname
#	same above except for view only by $PAGER (default=less)
#
# Requirement(s)
#   diffuN.sh
#
# Author(s)
#   Taiji Yamada <taiji@aihara.co.jp>
#

codename=${CODENAME:-`uname -n | sed -ne 's/\([^.]*\).*/\1/p' | tr A-Z a-z`}
patch_append=false; patch_backup=false; prepare_append=false
while [ "$1" != "" ]; do case "$1" in
-a) patch_append=:	;;
-b) patch_backup=:	;;
-p) prepare_append=:	;;
*) break	;;
esac; shift; done

pwd=`pwd`
basename=`basename "$pwd"`
patch="../$basename-$codename.patch"
prepare="../$basename-$codename-prepare.sh"

keep_file(){
  keep_file="$1"
  kept_file=
  if [ -f "$keep_file" ]; then
    kept_file="${keep_file}~"
    while [ -f "$kept_file" ]; do kept_file="${kept_file}~"; done
    echo "mv $keep_file $kept_file"
    mv "$keep_file" "$kept_file"
  fi
}
copy_file(){
  keep_file "$1"
  [ "$kept_file" = "" ] || cp "$kept_file" "$1"
}

patches(){
  if [ "$#" = 0 ]; then
    if [ -d "../$basename.orig" ]; then
      diffuN.sh "../$basename.orig" ./
    else
      if [ -d "../$basename.org" ]; then
        diffuN.sh "../$basename.org" ./
      else
        if [ -d "../$basename~" ]; then
          diffuN.sh "../$basename~" ./
        else
          diffuN.sh .
        fi
      fi
    fi
  else
    if [ "$#" = 1 -a -d "$1" ]; then
      diffuN.sh "$1" ./
    else
      diffuN.sh "$@"
    fi
  fi
}

[ $patch_append != false -a ! -f "$patch" ] && patch_append=false

case `basename $0 .sh` in
make-*)
  if [ "$patch_append" = false ]; then
    $patch_backup && keep_file "$patch"
    patches "$@" > "$patch"
  else
    $patch_backup && copy_file "$patch"
    patches "$@" >> "$patch"
  fi
  ${PAGER:-less} "$patch"
  if [ -f "$prepare" -a "$prepare_append" != false ]; then
    fgrep "$patch" "$prepare" > /dev/null 2>&1 || cat <<EOF >> "$prepare"
(cd $basename &&
patch -p0 -b -z.org < $patch
)
EOF
  fi
  ;;
view-*)
  if [ "$patch_append" = false ]; then
    patches "$@" | ${PAGER:-less}
  else
    (cat "$patch"; patches "$@") | ${PAGER:-less}
  fi
  ;;
esac
